/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.usercontrols;

import java.util.Collection;
import java.util.Iterator;
import java.util.TreeSet;

import ca.uvic.csr.shrimp.PanelModeConstants;
import ca.uvic.csr.shrimp.DataDisplayBridge.DataDisplayBridge;
import ca.uvic.csr.shrimp.DisplayBean.DisplayBean;
import ca.uvic.csr.shrimp.DisplayBean.DisplayConstants;
import ca.uvic.csr.shrimp.DisplayBean.ShrimpNode;
import ca.uvic.csr.shrimp.ShrimpApplication.BeanNotFoundException;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool;

/**
 * Shows all nodes immediately connected to the currently selected nodes by incoming or outgoing arcs.
 * All ancestors of these connected 
 * 
 * @author Rob Lintern, Chris Callendar
 */
public class ShowConnectedNodesAdapter extends ConnectedNodesAdapter {

	/**
	 * @param direction see {@link DisplayConstants}
	 */
	public ShowConnectedNodesAdapter(ShrimpProject project, ShrimpTool tool, String direction) {
		super("Show " + direction + " Nodes", project, tool, direction);
	}
	
	public void startAction() {
		try {
			DisplayBean displayBean = (DisplayBean) tool.getBean(ShrimpTool.DISPLAY_BEAN);
			DataDisplayBridge dataDisplayBridge = displayBean.getDataDisplayBridge();
			Collection connectedNodes = getConnectedNodes(true, false);
			Collection allAncestors = new TreeSet();
			for (Iterator iter = connectedNodes.iterator(); iter.hasNext();) {
				ShrimpNode node = (ShrimpNode) iter.next();
				allAncestors.addAll(dataDisplayBridge.getAncestorNodes(node));
			}
			for (Iterator iter = allAncestors.iterator(); iter.hasNext();) {
				ShrimpNode ancestorNode = (ShrimpNode) iter.next();
				displayBean.setPanelMode(ancestorNode, PanelModeConstants.CHILDREN);
			}
		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		} 
	}

}
